`timescale 1ns/1ps



/*
Implements a 512 x 128 memory using block rams (2048 words)
*/
module Ramb512x128(
	input CLK,
	input [8:0] Addr,
	input [127:0] DataIn,
	input WE,
	output [127:0] DataOut
) /* synthesis syn_black_box */ ;

//synopsys translate_off
	`library("word0","virtex2p")
	`library("word1","virtex2p")
	`library("word2","virtex2p")
	`library("word3","virtex2p")

	wor [3:0] parity_out;
	
	RAMB16_S36 word0
	(.CLK(CLK),
	.WE(WE),
	.SSR(1'b0),
	.EN(1'b1),
	.ADDR(Addr),
	.DIP(4'b0),
	.DI(DataIn[31:0]),
	.DO(DataOut[31:0]),
	.DOP(parity_out)
	);
	
	RAMB16_S36 word1
	(.CLK(CLK),
	.WE(WE),
	.SSR(1'b0),
	.EN(1'b1),
	.ADDR(Addr),
	.DIP(4'b0),
	.DI(DataIn[63:32]),
	.DO(DataOut[63:32]),
	.DOP(parity_out)
	);
	
	RAMB16_S36 word2
	(.CLK(CLK),
	.WE(WE),
	.SSR(1'b0),
	.EN(1'b1),
	.ADDR(Addr),
	.DIP(4'b0),
	.DI(DataIn[95:64]),
	.DO(DataOut[95:64]),
	.DOP(parity_out)
	);
	
	RAMB16_S36 word3
	(.CLK(CLK),
	.WE(WE),
	.SSR(1'b0),
	.EN(1'b1),
	.ADDR(Addr),
	.DIP(4'b0),
	.DI(DataIn[127:96]),
	.DO(DataOut[127:96]),
	.DOP(parity_out)
	);
	
//synopsys translate_on
	
endmodule	

/*
Implements a 2048 x 32 memory using block rams (2048 words)
*/
module Ramb2048x32(
	input CLK,
	input [10:0] Addr,
	input [31:0] DataIn,
	input WE,
	output [31:0] DataOut
	) /* synthesis syn_black_box */;
	
//synopsys translate_off
	`library("word0","virtex2p")
	`library("word1","virtex2p")
	`library("word2","virtex2p")
	`library("word3","virtex2p")
	
	wire S00, S01, S10, S11;
	wire WE00, WE01, WE10, WE11;
	wire [31:0] Data00, Data01, Data10, Data11;
	
	assign S00 = (Addr[10:9]==2'b00);
	assign S01 = (Addr[10:9]==2'b01);
	assign S10 = (Addr[10:9]==2'b10);
	assign S11 = (Addr[10:9]==2'b11);
	
	assign WE00 = WE && S00;
	assign WE01 = WE && S01;
	assign WE10 = WE && S10;
	assign WE11 = WE && S11;
	
	assign DataOut = S00 ? Data00 : (S01 ? Data01 : (S10 ? Data10 : Data11));
	wor [3:0] parity_out;
	
	RAMB16_S36 word0
	(.CLK(CLK),
	.WE(WE && (Addr[10:9]==2'b00)),
	.SSR(1'b0),
	.EN(1'b1),
	.ADDR(Addr[8:0]),
	.DIP(4'b0),
	.DI(DataIn),
	.DO(Data00),
	.DOP(parity_out)
	);
	
	RAMB16_S36 word1
	(.CLK(CLK),
	.WE(WE),
	.SSR(1'b0),
	.EN(1'b1),
	.ADDR(Addr[8:0]),
	.DIP(4'b0),
	.DI(DataIn),
	.DO(Data01),
	.DOP(parity_out)
	);
	
	RAMB16_S36 word2
	(.CLK(CLK),
	.WE(WE),
	.SSR(1'b0),
	.EN(1'b1),
	.ADDR(Addr[8:0]),
	.DIP(4'b0),
	.DI(DataIn),
	.DO(Data10),
	.DOP(parity_out)
	);
	
	RAMB16_S36 word3
	(.CLK(CLK),
	.WE(WE),
	.SSR(1'b0),
	.EN(1'b1),
	.ADDR(Addr[8:0]),
	.DIP(4'b0),
	.DI(DataIn),
	.DO(Data11),
	.DOP(parity_out)
	);
	
//synopsys_translate_on
	
endmodule



module CacheBIOSROM64x128(
	input CLK,
	input [5:0] ADDR,
	output [127:0] DataOut
	) /* synthesis syn_black_box */ ;
	
	//synopsys translate_off
	reg [127:0] OutputDelay;
	assign DataOut = OutputDelay;
	
	localparam CACHE_LINES = 27;
	wire [127:0] rom [CACHE_LINES-1:0];
	
	always @(posedge CLK)
		if (ADDR < CACHE_LINES)
			OutputDelay <= rom[ADDR];
		else
			OutputDelay <= 127'b0;
	
	//rom hex dump	      --byte3- --byte2-	--byte1- --byte0-
	assign rom[0] =  128'hae110000_34110001_37300040_3c198000;
	assign rom[1] =  128'h93290001_00000000_1100fffe_93280000;
	assign rom[2] =  128'h00000000_13090011_20180000_a3200003;
	assign rom[3] =  128'h23180001_00000000_13090030_23180001;
	assign rom[4] =  128'h13090019_23180001_00000000_1309003c;
	assign rom[5] =  128'h93280000_00000000_10000050_00000000;
	assign rom[6] =  128'h00000000_03e00008_00000000_1100fffe;
	assign rom[7] =  128'h0c000017_200b0000_200d0000_200c0004;
	assign rom[8] =  128'h01af7804_a3200003_932f0001_00000000;
	assign rom[9] =  128'h1580fff7_21ad0008_218cffff_016f5825;
	assign rom[10] = 128'h1000ffd8_214a0004_ad4b0000_00000000;
	assign rom[11] = 128'h200b0000_200d0000_200c0004_00000000;
	assign rom[12] = 128'ha3200003_932f0001_00000000_0c000017;
	assign rom[13] = 128'h21ad0008_218cffff_016f5825_01af7804;
	assign rom[14] = 128'h214a0001_a14b0000_00000000_1580fff7;
	assign rom[15] = 128'h200d0000_200c0004_00000000_1000ffc7;
	assign rom[16] = 128'h932f0001_00000000_0c000017_200a0000;
	assign rom[17] = 128'h218cffff_014f5025_01af7804_a3200003;
	assign rom[18] = 128'h1000ffb8_00000000_1580fff7_21ad0008;
	assign rom[19] = 128'ha3200003_00000000_0c000017_00000000;
	assign rom[20] = 128'h0c000017_a3200003_00000000_0c000017;
	assign rom[21] = 128'h00000000_0c000017_a3200003_00000000;
	assign rom[22] = 128'h340e0100_3c1d7fff_3c090040_a3200003;
	assign rom[23] = 128'h15c0fffc_21ceffff_23bd0004_8fa00000;
	assign rom[24] = 128'hae000000_37bdfffc_3c1d7fff_00000000;
	assign rom[25] = 128'hae110000_36310008_00000000_01200008;
	assign rom[26] = 128'h00000000_00000000_00000000_1000ffff;
	
//synopsys translate_on
endmodule


module CacheBIOSROM256x32(
	input CLK,
	input [7:0] ADDR,
	output [31:0] DataOut
) /* synthesis syn_black_box */ ;
//synopsys translate_off
	wire [127:0] rom;
	
	CacheBIOSROM64x128 wbios (
	.CLK(CLK),
	.ADDR(ADDR[7:2]),
	.DataOut(rom));
	
	reg [1:0] word;
	//assign word = ADDR[1:0];
	assign DataOut = (word==2'b00 ? rom[31:0] : (word==2'b01 ? rom[63:32] :	 (word==2'b10 ? rom[95:64] : rom [127:96])));
	
	always @(posedge CLK)
		word = ADDR[1:0];
		
//synopsys translate_on
endmodule

module IOMemory(input [31:0] Addr, input Read, Write, output InvalidAddress, output IODev0_SEL, IODev1_SEL, IODev2_SEL, IODev3_SEL, IOVGAChar_SEL, IOVGAColor_SEL
	) /* synthesis syn_black_box */ ;
	//synopsys translate_off
	
	localparam IODEV_ADDR = 24'h8000_00,
	VGA_CHAR_ADDR = 19'b1000_0000_0000_0000_110,
	VGA_COLOR_ADDR = 19'b1000_0000_0000_0000_111;
	
	assign IODev0_SEL = (Addr[31:8] == IODEV_ADDR) && (Addr[7:6]==2'b00);
	assign IODev1_SEL = (Addr[31:8] == IODEV_ADDR) && (Addr[7:6]==2'b01);
	assign IODev2_SEL = (Addr[31:8] == IODEV_ADDR) && (Addr[7:6]==2'b10);
	assign IODev3_SEL = (Addr[31:8] == IODEV_ADDR) && (Addr[7:6]==2'b11);
	assign IOVGAChar_SEL = (Addr[31:13] == VGA_CHAR_ADDR);
	assign IOVGAColor_SEL = (Addr[31:13] == VGA_COLOR_ADDR);
	
	assign InvalidAddress = (Read || Write) && ~(IODev0_SEL || IODev1_SEL || IODev2_SEL || IODev3_SEL || IOVGAChar_SEL || IOVGAColor_SEL);
	
//synopsys translate_on
endmodule

// -------- Legacy Memories -------

// -------- Ram256x32_sw_ar_wb.v --------

module RamBank256_8_sw_ar(
	input wire [7:0] ADDR,
	input wire [7:0] DI,
	output wire [7:0] DO,
	input wire CLK,
	input wire WE
) ;
	// ----------------- Constants ------------ //
	parameter DANGLING_INPUT_CONSTANT = 1'bZ;
    parameter INIT_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_04 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_05 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_06 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_07 = 256'h0000000000000000000000000000000000000000000000000000000000000000;

	// ----------- Signal declarations -------- //
	wire [7:0] O_0;
	wire [7:0] O_1;
	wire [7:0] O_2;
	wire [7:0] O_3;
	wire [7:0] O_4;
	wire [7:0] O_5;
	wire [7:0] O_6;
	wire [7:0] O_7;
	
	wire [7:0] WRITE;
	
	
	// ---- Declaration for Dangling inputs ----//
	wire Dangling_Input_Signal = DANGLING_INPUT_CONSTANT;
	
	// -------- Component instantiations -------//
	
	assign DO = ((ADDR[7])? ((ADDR[6])? ((ADDR[5])? O_7 : O_6) : ((ADDR[5])? O_5 : O_4)) : ((ADDR[6])? ((ADDR[5])? O_3 : O_2) : ((ADDR[5])? O_1 : O_0)));
	assign WRITE = { (ADDR[7:5] == 3'h7),(ADDR[7:5] == 3'h6),(ADDR[7:5] == 3'h5),(ADDR[7:5] == 3'h4),(ADDR[7:5] == 3'h3),(ADDR[7:5] == 3'h2),(ADDR[7:5] == 3'h1),(ADDR[7:5] == 3'h0)};	
	
//synopsys translate_off

`library("U1","ovi_unisim")
`library("U2","ovi_unisim")
`library("U3","ovi_unisim")
`library("U4","ovi_unisim")
`library("U5","ovi_unisim")
`library("U6","ovi_unisim")
`library("U7","ovi_unisim")
`library("U8","ovi_unisim")
//synopsys translate_on

	RAM32X8S U1
		(
		.A0(ADDR[0]),
		.A1(ADDR[1]),
		.A2(ADDR[2]),
		.A3(ADDR[3]),
		.A4(ADDR[4]),
		.D(DI),
		.O(O_0),
		.WCLK(CLK),
		.WE(WRITE[0]&WE)
		);
	
	
	RAM32X8S U2
		(
		.A0(ADDR[0]),
		.A1(ADDR[1]),
		.A2(ADDR[2]),
		.A3(ADDR[3]),
		.A4(ADDR[4]),
		.D(DI),
		.O(O_1),
		.WCLK(CLK),
		.WE(WRITE[1]&WE)
		);
	
	RAM32X8S U3
		(
		.A0(ADDR[0]),
		.A1(ADDR[1]),
		.A2(ADDR[2]),
		.A3(ADDR[3]),
		.A4(ADDR[4]),
		.D(DI),
		.O(O_2),
		.WCLK(CLK),
		.WE(WRITE[2]&WE)
		);

	RAM32X8S U4
		(
		.A0(ADDR[0]),
		.A1(ADDR[1]),
		.A2(ADDR[2]),
		.A3(ADDR[3]),
		.A4(ADDR[4]),
		.D(DI),
		.O(O_3),
		.WCLK(CLK),
		.WE(WRITE[3]&WE)
		);

	RAM32X8S U5
		(
		.A0(ADDR[0]),
		.A1(ADDR[1]),
		.A2(ADDR[2]),
		.A3(ADDR[3]),
		.A4(ADDR[4]),
		.D(DI),
		.O(O_4),
		.WCLK(CLK),
		.WE(WRITE[4]&WE)
		);
	
	RAM32X8S U6
		(
		.A0(ADDR[0]),
		.A1(ADDR[1]),
		.A2(ADDR[2]),
		.A3(ADDR[3]),
		.A4(ADDR[4]),
		.D(DI),
		.O(O_5),
		.WCLK(CLK),
		.WE(WRITE[5]&WE)
		);
	
	RAM32X8S U7
		(
		.A0(ADDR[0]),
		.A1(ADDR[1]),
		.A2(ADDR[2]),
		.A3(ADDR[3]),
		.A4(ADDR[4]),
		.D(DI),
		.O(O_6),
		.WCLK(CLK),
		.WE(WRITE[6]&WE)
		);
	
	RAM32X8S U8
		(
		.A0(ADDR[0]),
		.A1(ADDR[1]),
		.A2(ADDR[2]),
		.A3(ADDR[3]),
		.A4(ADDR[4]),
		.D(DI),
		.O(O_7),
		.WCLK(CLK),
		.WE(WRITE[7]&WE)
		);

endmodule 

module ram_control ( 
	input wire WE,
	input wire [1:0] SIZE ,
	input wire [1:0] ADDR ,
	output wire [3:0] BankWE
) /* synthesis syn_black_box */ ;
//synopsys translate_off

wire WW,WB,WH;
// SIZE = 2'b00  :  Don't write
//      = 2'b01  :  Write Byte
//      = 2'b10  :  Write half-Word
//      = 2'b11  :  Write Word

assign WW = (SIZE == 2'b11);
assign WB = (SIZE == 2'b01);
assign WH = (SIZE == 2'b10);

// -------- Component instantiations -------//
assign BankWE[0] = WE & ( WW | (WB & (ADDR[1:0] == 2'b00)) |(WH & ~ADDR[1]));
assign BankWE[1] = WE & ( WW | (WB & (ADDR[1:0] == 2'b01)) |(WH & ~ADDR[1]));
assign BankWE[2] = WE & ( WW | (WB & (ADDR[1:0] == 2'b10)) |(WH & ADDR[1]));
assign BankWE[3] = WE & ( WW | (WB & (ADDR[1:0] == 2'b11)) |(WH & ADDR[1]));

//synopsys translate_on
endmodule

module store_align(
input wire [1:0] SIZE,
input wire [31:0] DI,
output wire [31:0] DO
) /* synthesis syn_black_box */;
//synopsys translate_off

wire WW,WB,WH;
wire [7:0] Byte3_DI, Byte2_DI, Byte1_DI;

assign WW = (SIZE == 2'b11);
assign WB = (SIZE == 2'b01);
assign WH = (SIZE == 2'b10);	
	
assign Byte3_DI = (WB) ? DI[7:0] : (WH) ? DI[15:8] : DI[31:24];
assign Byte2_DI = (WW) ? DI[23:16] : DI[7:0];
assign Byte1_DI = (WB) ? DI[7:0] : DI[15:8];	

assign DO = {Byte3_DI,Byte2_DI,Byte1_DI,DI[7:0]};
//synopsys translate_on

endmodule	

module Ram256x32_sw_ar_wb (
	input wire CLK,
	input wire [9:0] ADDR,
	input wire [31:0] DI,
	input wire WE,
	input wire [1:0] SIZE,
	output wire [31:0] DO
) ;
	parameter INIT_FILE="$DSN\\src\\zero.hex";

`ifdef SYNTHESIS
// --- User defined diagram declarations -- //
// Enter your code here
	// Parameters used to init the internal ROMs
	parameter INIT = 32'h0;
	
	parameter INIT_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_04 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_05 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_06 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_07 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_08 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_09 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_0A = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_0B = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_0C = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_0D = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_0E = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_0F = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_10 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_11 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_12 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_13 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_14 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_15 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_16 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_17 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_18 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_19 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_1A = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_1B = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_1C = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_1D = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_1E = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_1F = 256'h0000000000000000000000000000000000000000000000000000000000000000;

`endif
// ------------ Port declarations --------- //


// ----------- Signal declarations -------- //
wire [3:0] BWE;
wire WW,WH,WB;

wire [31:0] AlignDI;

// SIZE = 2'b00  :  Don't write
//      = 2'b01  :  Write Byte
//      = 2'b10  :  Write half-Word
//      = 2'b11  :  Write Word

assign WW = (SIZE == 2'b11);
assign WB = (SIZE == 2'b01);
assign WH = (SIZE == 2'b10);

// -------- Component instantiations -------//
ram_control rc( .WE(WE), .SIZE(SIZE), .ADDR(ADDR[1:0]),.BankWE(BWE));
store_align sa( .SIZE(SIZE), .DI(DI), .DO(AlignDI));


`ifdef SYNTHESIS
RamBank256_8_sw_ar Byte0
(
	.ADDR(ADDR[9:2]),
	.CLK(CLK),
	.DI(AlignDI[7:0]),
	.DO(DO[7:0]),
	.WE(BWE[0])
);

RamBank256_8_sw_ar Byte1
(
	.ADDR(ADDR[9:2]),
	.CLK(CLK),
	.DI(AlignDI[15:8]),
	.DO(DO[15:8]),
	.WE(BWE[1])
);

RamBank256_8_sw_ar Byte2
(
	.ADDR(ADDR[9:2]),
	.CLK(CLK),
	.DI(AlignDI[23:16]),
	.DO(DO[23:16]),
	.WE(BWE[2])
);


RamBank256_8_sw_ar Byte3
(
	.ADDR(ADDR[9:2]),
	.CLK(CLK),
	.DI(AlignDI[31:24]),
	.DO(DO[31:24]),
	.WE(BWE[3])
);
`else
	reg [31:0] RAM [0:255];
	
	assign DO = RAM[ADDR[9:2]];
	
	initial begin
		$readmemh(INIT_FILE,RAM);
	end
	
	always @(posedge CLK) begin
		if(WE) begin
			RAM[ADDR[9:2]] <= {((BWE[3])? AlignDI[31:24] : DO[31:24]),
							((BWE[2])? AlignDI[23:16] : DO[23:16]),
							((BWE[1])? AlignDI[15:8] : DO[15:8]),
							((BWE[0])? AlignDI[7:0] : DO[7:0])}; 
	    end
	end
`endif	
	
// ----------- Terminals assignment --------//

endmodule 

// -------- ramb16_s36_wb.v ----------

module ramb16_s36_wb (
	input wire CLK,
	input wire EN,
	input wire SSR,
	input wire [12:0] ADDR,
	input wire [31:0] DI,
	input wire [3:0] DIP,
	input wire WE,
	input wire [1:0] SIZE,
	output wire [31:0] DO,
	output wire [3:0] DOP
) /* synthesis syn_black_box */ ;
//synopsys translate_off

// --- User defined diagram declarations -- //
// Enter your code here
	// Parameters used to init the internal ROMs
	parameter INIT = 36'h0;
	parameter SRVAL = 36'h0;
	parameter WRITE_MODE = "WRITE_FIRST";
	
	parameter INIT_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_04 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_05 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_06 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_07 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_08 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_09 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_0A = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_0B = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_0C = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_0D = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_0E = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_0F = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_10 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_11 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_12 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_13 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_14 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_15 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_16 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_17 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_18 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_19 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_1A = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_1B = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_1C = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_1D = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_1E = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_1F = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_20 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_21 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_22 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_23 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_24 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_25 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_26 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_27 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_28 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_29 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_2A = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_2B = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_2C = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_2D = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_2E = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_2F = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_30 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_31 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_32 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_33 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_34 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_35 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_36 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_37 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_38 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_39 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_3A = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_3B = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_3C = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_3D = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_3E = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INIT_3F = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INITP_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INITP_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INITP_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INITP_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INITP_04 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INITP_05 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INITP_06 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	parameter INITP_07 = 256'h0000000000000000000000000000000000000000000000000000000000000000;



// ----------- Signal declarations -------- //
wire [3:0] BWE;

// -------- Component instantiations -------//
`library("Byte0","ovi_unisim")
`library("Byte1","ovi_unisim")
`library("Byte2","ovi_unisim")
`library("Byte3","ovi_unisim")

RAMB16_S9 Byte0
(
	.ADDR(ADDR[12:2]),
	.CLK(CLK),
	.DI(DI[7:0]),
	.DIP(DIP[0:0]),
	.DO(DO[7:0]),
	.DOP(DOP[0:0]),
	.EN(EN),
	.SSR(SSR),
	.WE(BWE[0])
);

RAMB16_S9 Byte1
(
	.ADDR(ADDR[12:2]),
	.CLK(CLK),
	.DI(DI[15:8]),
	.DIP(DIP[1:1]),
	.DO(DO[15:8]),
	.DOP(DOP[1:1]),
	.EN(EN),
	.SSR(SSR),
	.WE(BWE[1])
);

RAMB16_S9 Byte2
(
	.ADDR(ADDR[12:2]),
	.CLK(CLK),
	.DI(DI[23:16]),
	.DIP(DIP[2:2]),
	.DO(DO[23:16]),
	.DOP(DOP[2:2]),
	.EN(EN),
	.SSR(SSR),
	.WE(BWE[2])
);

RAMB16_S9 Byte3
(
	.ADDR(ADDR[12:2]),
	.CLK(CLK),
	.DI(DI[31:24]),
	.DIP(DIP[3:3]),
	.DO(DO[31:24]),
	.DOP(DOP[3:3]),
	.EN(EN),
	.SSR(SSR),
	.WE(BWE[3])
);

ram_control U1
(
	.WE(WE),
	.SIZE(SIZE),
	.ADDR(ADDR[1:0]),
	.BankWE(BWE)
);



// ----------- Terminals assignment --------//
//	       ---- Input terminals ---         //

//synopsys translate_on
endmodule 

// ---------- imemory.v ------------

module ROM256x32 (
	input wire [7:0] ADDR,
	output wire [31:0] OUTPUT
	) /* synthesis syn_black_box */ ; 
	//synopsys translate_off

	// --- User defined diagram declarations -- //
	// Parameters used to init the internal ROMs
	
	parameter
		INIT_C0 = 256'h0000000000000000000000000000000000000000000000000000000000000043,
		INIT_C1 = 256'h0000000000000000000000000000000000000000000000000000000000000040,
		INIT_C2 = 256'h0000000000000000000000000000000000000000000000000000000000000040,
		INIT_C3 = 256'h0000000000000000000000000000000000000000000000000000000000000040,
		INIT_C4 = 256'h0000000000000000000000000000000000000000000000000000000000000043,
		INIT_C5 = 256'h0000000000000000000000000000000000000000000000000000000000000043,
		INIT_C6 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
		INIT_C7 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
		INIT_C8 = 256'h000000000000000000000000000000000000000000000000000000000000003c,
		INIT_C9 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
		INIT_C10 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
		INIT_C11 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
		INIT_C12 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
		INIT_C13 = 256'h000000000000000000000000000000000000000000000000000000000000003f,
		INIT_C14 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
		INIT_C15 = 256'h000000000000000000000000000000000000000000000000000000000000003e,
		INIT_C16 = 256'h000000000000000000000000000000000000000000000000000000000000003c,
		INIT_C17 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
		INIT_C18 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
		INIT_C19 = 256'h0000000000000000000000000000000000000000000000000000000000000030,
		INIT_C20 = 256'h0000000000000000000000000000000000000000000000000000000000000028,
		INIT_C21 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
		INIT_C22 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
		INIT_C23 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
		INIT_C24 = 256'h0000000000000000000000000000000000000000000000000000000000000003,
		INIT_C25 = 256'h0000000000000000000000000000000000000000000000000000000000000003,
		INIT_C26 = 256'h000000000000000000000000000000000000000000000000000000000000003f,
		INIT_C27 = 256'h0000000000000000000000000000000000000000000000000000000000000003,
		INIT_C28 = 256'h0000000000000000000000000000000000000000000000000000000000000003,
		INIT_C29 = 256'h0000000000000000000000000000000000000000000000000000000000000033,
		INIT_C30 = 256'h000000000000000000000000000000000000000000000000000000000000000a,
		INIT_C31 = 256'h0000000000000000000000000000000000000000000000000000000000000011;
	
	defparam
		U1_ARRAY0.INIT = INIT_C0, 
		U1_ARRAY1.INIT = INIT_C1, 
		U1_ARRAY2.INIT = INIT_C2, 
		U1_ARRAY3.INIT = INIT_C3, 
		U1_ARRAY4.INIT = INIT_C4, 
		U1_ARRAY5.INIT = INIT_C5, 
		U1_ARRAY6.INIT = INIT_C6, 
		U1_ARRAY7.INIT = INIT_C7, 
		U1_ARRAY8.INIT = INIT_C8, 
		U1_ARRAY9.INIT = INIT_C9, 
		U1_ARRAY10.INIT = INIT_C10,
		U1_ARRAY11.INIT = INIT_C11,
		U1_ARRAY12.INIT = INIT_C12,
		U1_ARRAY13.INIT = INIT_C13,
		U1_ARRAY14.INIT = INIT_C14,
		U1_ARRAY15.INIT = INIT_C15,
		U1_ARRAY16.INIT = INIT_C16,
		U1_ARRAY17.INIT = INIT_C17,
		U1_ARRAY18.INIT = INIT_C18,
		U1_ARRAY19.INIT = INIT_C19,
		U1_ARRAY20.INIT = INIT_C20,
		U1_ARRAY21.INIT = INIT_C21,
		U1_ARRAY22.INIT = INIT_C22,
		U1_ARRAY23.INIT = INIT_C23,
		U1_ARRAY24.INIT = INIT_C24,
		U1_ARRAY25.INIT = INIT_C25,
		U1_ARRAY26.INIT = INIT_C26,
		U1_ARRAY27.INIT = INIT_C27,
		U1_ARRAY28.INIT = INIT_C28,
		U1_ARRAY29.INIT = INIT_C29,
		U1_ARRAY30.INIT = INIT_C30,
		U1_ARRAY31.INIT = INIT_C31;
	
	// -------- Component instantiations -------//
	reg [31:0] Fake [0:255];
	integer i; 
	integer j; 
	reg [31:0] Temp;
	initial begin
			for(i = 0; i < 256; i = i + 1) begin
					Temp[0] = U1_ARRAY0.INIT[i];		
					Temp[1] = U1_ARRAY1.INIT[i];		
					Temp[2] = U1_ARRAY2.INIT[i];		
					Temp[3] = U1_ARRAY3.INIT[i];		
					Temp[4] = U1_ARRAY4.INIT[i];		
					Temp[5] = U1_ARRAY5.INIT[i];		
					Temp[6] = U1_ARRAY6.INIT[i];		
					Temp[7] = U1_ARRAY7.INIT[i];		
					Temp[8] = U1_ARRAY8.INIT[i];		
					Temp[9] = U1_ARRAY9.INIT[i];		
					Temp[10] = U1_ARRAY10.INIT[i];		
					Temp[11] = U1_ARRAY11.INIT[i];		
					Temp[12] = U1_ARRAY12.INIT[i];		
					Temp[13] = U1_ARRAY13.INIT[i];		
					Temp[14] = U1_ARRAY14.INIT[i];		
					Temp[15] = U1_ARRAY15.INIT[i];		
					Temp[16] = U1_ARRAY16.INIT[i];		
					Temp[17] = U1_ARRAY17.INIT[i];		
					Temp[18] = U1_ARRAY18.INIT[i];		
					Temp[19] = U1_ARRAY19.INIT[i];		
					Temp[20] = U1_ARRAY20.INIT[i];		
					Temp[21] = U1_ARRAY21.INIT[i];		
					Temp[22] = U1_ARRAY22.INIT[i];		
					Temp[23] = U1_ARRAY23.INIT[i];		
					Temp[24] = U1_ARRAY24.INIT[i];		
					Temp[25] = U1_ARRAY25.INIT[i];		
					Temp[26] = U1_ARRAY26.INIT[i];		
					Temp[27] = U1_ARRAY27.INIT[i];		
					Temp[28] = U1_ARRAY28.INIT[i];		
					Temp[29] = U1_ARRAY29.INIT[i];		
					Temp[30] = U1_ARRAY30.INIT[i];		
					Temp[31] = U1_ARRAY31.INIT[i];		
					Fake[i] = Temp;
				end
		end

	`library("U1_ARRAY0","ovi_unisim")
	`library("U1_ARRAY1","ovi_unisim")
	`library("U1_ARRAY2","ovi_unisim")
	`library("U1_ARRAY3","ovi_unisim")
	`library("U1_ARRAY4","ovi_unisim")
	`library("U1_ARRAY5","ovi_unisim")
	`library("U1_ARRAY6","ovi_unisim")
	`library("U1_ARRAY7","ovi_unisim")
	`library("U1_ARRAY8","ovi_unisim")
	`library("U1_ARRAY9","ovi_unisim")
	`library("U1_ARRAY10","ovi_unisim")
	`library("U1_ARRAY11","ovi_unisim")
	`library("U1_ARRAY12","ovi_unisim")
	`library("U1_ARRAY13","ovi_unisim")
	`library("U1_ARRAY14","ovi_unisim")
	`library("U1_ARRAY15","ovi_unisim")
	`library("U1_ARRAY16","ovi_unisim")
	`library("U1_ARRAY17","ovi_unisim")
	`library("U1_ARRAY18","ovi_unisim")
	`library("U1_ARRAY19","ovi_unisim")
	`library("U1_ARRAY20","ovi_unisim")
	`library("U1_ARRAY21","ovi_unisim")
	`library("U1_ARRAY22","ovi_unisim")
	`library("U1_ARRAY23","ovi_unisim")
	`library("U1_ARRAY24","ovi_unisim")
	`library("U1_ARRAY25","ovi_unisim")
	`library("U1_ARRAY26","ovi_unisim")
	`library("U1_ARRAY27","ovi_unisim")
	`library("U1_ARRAY28","ovi_unisim")
	`library("U1_ARRAY29","ovi_unisim")
	`library("U1_ARRAY30","ovi_unisim")
	`library("U1_ARRAY31","ovi_unisim")

	ROM256X1 U1_ARRAY0
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[0]})
		);

	ROM256X1 U1_ARRAY1
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[1]})
		);

	ROM256X1 U1_ARRAY2
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[2]})
		);

	ROM256X1 U1_ARRAY3
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[3]})
		);

	ROM256X1 U1_ARRAY4
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[4]})
		);
	
	ROM256X1 U1_ARRAY5
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[5]})
		);
	
	ROM256X1 U1_ARRAY6
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[6]})
		);
	
	ROM256X1 U1_ARRAY7
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[7]})
		);
	
	ROM256X1 U1_ARRAY8
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[8]})
		);
	
	ROM256X1 U1_ARRAY9
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[9]})
		);
	
	ROM256X1 U1_ARRAY10
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[10]})
		);
	
	ROM256X1 U1_ARRAY11
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[11]})
		);
	
	ROM256X1 U1_ARRAY12
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[12]})
		);
	
	ROM256X1 U1_ARRAY13
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[13]})
		);
	
	ROM256X1 U1_ARRAY14
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[14]})
		);
	
	ROM256X1 U1_ARRAY15
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[15]})
		);
	
	ROM256X1 U1_ARRAY16
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[16]})
		);
	
	ROM256X1 U1_ARRAY17
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[17]})
		);
	
	ROM256X1 U1_ARRAY18
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[18]})
		);
	
	ROM256X1 U1_ARRAY19
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[19]})
		);
	
	ROM256X1 U1_ARRAY20
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[20]})
		);
	
	ROM256X1 U1_ARRAY21
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[21]})
		);
	
	ROM256X1 U1_ARRAY22
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[22]})
		);
	
	ROM256X1 U1_ARRAY23
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[23]})
		);
	
	ROM256X1 U1_ARRAY24
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[24]})
		);
	
	ROM256X1 U1_ARRAY25
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[25]})
		);
	
	ROM256X1 U1_ARRAY26
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[26]})
		);
	
	ROM256X1 U1_ARRAY27
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[27]})
		);
	
	ROM256X1 U1_ARRAY28
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[28]})
		);
	
	ROM256X1 U1_ARRAY29
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[29]})
		);
	
	ROM256X1 U1_ARRAY30
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[30]})
		);
	
	ROM256X1 U1_ARRAY31
		(
		.A0({ADDR[0]}),
		.A1({ADDR[1]}),
		.A2({ADDR[2]}),
		.A3({ADDR[3]}),
		.A4({ADDR[4]}),
		.A5({ADDR[5]}),
		.A6({ADDR[6]}),
		.A7({ADDR[7]}),
		.O({OUTPUT[31]})
		);

//synopsys translate_on
		
endmodule 








